function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

import { Module } from './Module';
import { TokenObserver } from '../observers';
export var Manager =
/*#__PURE__*/
function () {
  function Manager(application) {
    _classCallCheck(this, Manager);

    this.application = application;
    this.observer = new TokenObserver(this.element, this.componentAttribute, this);
    this.modulesBySlug = new Map();
  }

  _createClass(Manager, [{
    key: "start",
    value: function start() {
      this.observer.start();
    }
  }, {
    key: "stop",
    value: function stop() {
      this.observer.stop();
    }
  }, {
    key: "addModule",
    value: function addModule(definition) {
      var noElement = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var args = arguments.length > 2 ? arguments[2] : undefined;
      var slug = definition.slug;
      this.removeModule(slug);
      var module = new Module(this.application, definition, args);
      this.modulesBySlug.set(slug, module); // Init module

      this._initModule(module, noElement);
    }
  }, {
    key: "removeModule",
    value: function removeModule(slug) {
      var module = this.modulesBySlug.get(slug);

      if (module) {
        this.modulesBySlug.delete(slug); // Destroy module

        this._destroyModule(module);
      }
    } // Token observer delegate

  }, {
    key: "elementMatchedToken",
    value: function elementMatchedToken(element, token) {
      this._initModuleBySlug(token, element);
    }
  }, {
    key: "elementUnmatchedToken",
    value: function elementUnmatchedToken(element, token) {
      this._destroyModuleBySlug(token, element);
    } // Contexts

  }, {
    key: "_initModule",
    // Private
    value: function _initModule(module, noElement) {
      var elements = this.observer.getElementsMatchingToken(module.slug); // NoElement means no DOM element (so no load and lifecycle)
      // Args are used only for "mix" [no]components

      if (noElement) {
        module.initNoElement(module.slug, module.args);
      } else {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = elements[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var element = _step.value;
            module.initElement(element, module.args);
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }
    }
  }, {
    key: "_destroyModule",
    value: function _destroyModule(module) {
      // eslint-disable-line class-methods-use-this
      var contexts = module.contexts;
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = contexts[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var element = _step2.value.element;
          module.destroyElement(element);
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }
  }, {
    key: "_initModuleBySlug",
    value: function _initModuleBySlug(slug, element) {
      var module = this.modulesBySlug.get(slug);

      if (module) {
        module.initElement(element, module.args);
      }
    }
  }, {
    key: "_destroyModuleBySlug",
    value: function _destroyModuleBySlug(slug, element) {
      var module = this.modulesBySlug.get(slug);

      if (module) {
        module.destroyElement(element);
      }
    }
  }, {
    key: "schema",
    get: function get() {
      return this.application.schema;
    }
  }, {
    key: "element",
    get: function get() {
      return this.application.element;
    }
  }, {
    key: "componentAttribute",
    get: function get() {
      return this.schema.componentAttribute;
    }
  }, {
    key: "modules",
    get: function get() {
      return Array.from(this.modulesBySlug.values());
    }
  }, {
    key: "contexts",
    get: function get() {
      return this.modules.reduce(function (contexts, module) {
        return contexts.concat(Array.from(module.contexts));
      }, []);
    }
  }]);

  return Manager;
}();