function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

export var ElementObserver =
/*#__PURE__*/
function () {
  function ElementObserver(element, delegate) {
    var _this = this;

    _classCallCheck(this, ElementObserver);

    this.element = element;
    this.delegate = delegate;
    this.started = false;
    this.elements = new Set();
    this.mutationObserver = new MutationObserver(function (mutations) {
      return _this._processMutations(mutations);
    });
  } // Observer / elements handling


  _createClass(ElementObserver, [{
    key: "start",
    value: function start() {
      if (!this.started) {
        this.mutationObserver.observe(this.element, {
          attributes: true,
          childList: true,
          subtree: true
        });
        this.started = true;
        this.refresh();
      }
    }
  }, {
    key: "stop",
    value: function stop() {
      if (this.started) {
        this.mutationObserver.takeRecords();
        this.mutationObserver.disconnect();
        this.started = false;
      }
    }
  }, {
    key: "refresh",
    value: function refresh() {
      if (this.started) {
        var matches = new Set(this._matchElementsInTree());

        var _arr = Array.from(this.elements);

        for (var _i = 0; _i < _arr.length; _i++) {
          var element = _arr[_i];

          if (!matches.has(element)) {
            this._removeElement(element);
          }
        }

        var _arr2 = Array.from(matches);

        for (var _i2 = 0; _i2 < _arr2.length; _i2++) {
          var _element = _arr2[_i2];

          this._addElement(_element);
        }
      }
    } // Delegate

  }, {
    key: "_matchElement",
    value: function _matchElement(element) {
      return this.delegate.matchElement(element);
    }
  }, {
    key: "_matchElementsInTree",
    value: function _matchElementsInTree() {
      var tree = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.element;
      return this.delegate.matchElementsInTree(tree);
    } // Observer callback / process

  }, {
    key: "_processMutations",
    value: function _processMutations(mutations) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = mutations[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var mutation = _step.value;

          this._processMutation(mutation);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
  }, {
    key: "_processMutation",
    value: function _processMutation(mutation) {
      if (mutation.type === 'attributes') {
        this._processAttributeChange(mutation.target, mutation.attributeName);
      } else if (mutation.type === 'childList') {
        this._processRemovedNodes(mutation.removedNodes);

        this._processAddedNodes(mutation.addedNodes);
      }
    }
  }, {
    key: "_processAttributeChange",
    value: function _processAttributeChange(node, attributeName) {
      var element = node;

      if (this.elements.has(element)) {
        if (this.delegate.elementAttributeChanged && this._matchElement(element)) {
          this.delegate.elementAttributeChanged(element, attributeName);
        } else {
          this._removeElement(element);
        }
      } else if (this._matchElement(element)) {
        this._addElement(element);
      }
    }
  }, {
    key: "_processRemovedNodes",
    value: function _processRemovedNodes(nodes) {
      var _arr3 = Array.from(nodes);

      for (var _i3 = 0; _i3 < _arr3.length; _i3++) {
        var node = _arr3[_i3];

        this._processNode(node, this._removeElement);
      }
    }
  }, {
    key: "_processAddedNodes",
    value: function _processAddedNodes(nodes) {
      var _arr4 = Array.from(nodes);

      for (var _i4 = 0; _i4 < _arr4.length; _i4++) {
        var node = _arr4[_i4];

        this._processNode(node, this._addElement);
      }
    }
  }, {
    key: "_processNode",
    value: function _processNode(node, processor) {
      var tree = ElementObserver.elementFromNode(node);

      if (tree) {
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = this._matchElementsInTree(tree)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var element = _step2.value;
            processor.call(this, element);
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      }
    } // Process only Element nodes

  }, {
    key: "_addElement",
    value: function _addElement(element) {
      if (!this.elements.has(element)) {
        this.elements.add(element);

        if (this.delegate.elementMatched) {
          this.delegate.elementMatched(element);
        }
      }
    }
  }, {
    key: "_removeElement",
    value: function _removeElement(element) {
      if (this.elements.has(element)) {
        this.elements.delete(element);

        if (this.delegate.elementUnmatched) {
          this.delegate.elementUnmatched(element);
        }
      }
    }
  }], [{
    key: "elementFromNode",
    value: function elementFromNode(node) {
      if (node.nodeType === Node.ELEMENT_NODE) {
        return node;
      }

      return false;
    }
  }]);

  return ElementObserver;
}();